// Generated by dts-bundle-generator v9.5.1

import Database from 'bun:sqlite';
import { Database } from 'bun:sqlite';

export type RedefineValueTypes<ShapeToRedefine, NewValueType> = {
	[K in keyof ShapeToRedefine]: NewValueType;
};
/** Types that define the data type of a column in the DB */
export type DataType = "TEXT" | "INTEGER" | "REAL" | "BLOB" | "NULL";
export interface Column {
	/** The data type to store this column */
	type: DataType;
	nullable: boolean;
	primary: boolean;
	/** Any additional freeform SQL */
	misc: string;
	/** If set, creates a foreign key association. See Table.reference() */
	references: string;
	cascade: boolean;
	default: string | number | undefined;
	unique: boolean;
	autoincrement: boolean;
}
export type ColumnShorthand = DataType | Partial<Column>;
export type ColumnShorthandMap<T> = RedefineValueTypes<T, ColumnShorthand>;
/**
 * Represents an instance of a row in a table of a database.
 * Wraps the raw Row data and provides methods for interacting with it.
 */
export declare class WrappedRow<RowShape> {
	protected row: RowShape;
	constructor(row: RowShape);
	/**
	 * @returns The row data as a JSON string.
	 */
	toJSON(): string;
	/**
	 * Exports the raw row data.
	 * @returns The raw row data.
	 */
	unwrap(): RowShape;
}
/** Types that may appear in a row */
export type ColumnValue = string | number | null;
/**
 * Represents a table in a database.
 *
 * @typeParam RowShape - The shape of the rows in the table.
 */
export declare class Table<RowShape> {
	private _name;
	private _db;
	private _columnDefinitions;
	private _columnNames;
	private _createTableSQL;
	private _primaryColumnName;
	private _insertQuery;
	private _updateQuery;
	private _findQuery;
	private _deleteQuery;
	private _deleteRowQuery;
	private _sizeQuery;
	private _truncateQuery;
	query: typeof Database.prototype.query;
	constructor(db: Database | Prequel, name: string, cols: ColumnShorthandMap<RowShape>);
	protected hasColumn(colName: string): boolean;
	/**
	 * Generates a reference string for a column in the table.
	 *
	 * @param colName - The name of the column to reference. If not provided, the primary column name is used.
	 * @returns A string representing the reference to the specified column or the primary column if no column name is provided.
	 * @throws {SchemaError} If the specified column name does not exist in the table schema.
	 */
	reference(colName?: keyof RowShape): string;
	/**
	 * Inserts a new row into the table.
	 *
	 * @param data - The data to be inserted, conforming to the RowShape interface.
	 * @returns The inserted row, conforming to the RowShape interface.
	 */
	insert(data: RowShape): RowShape;
	/**
	 * Inserts a partial row into the table and returns the inserted row.
	 *
	 * @param data - An object containing the partial data to be inserted. The keys should match the column names of the table.
	 * @returns The inserted row as an object of type `RowShape`.
	 *
	 * @typeParam RowShape - The shape of the row in the table.
	 */
	insertPartial(data: Partial<RowShape>): RowShape;
	insertWithout<OmitKeys extends keyof RowShape>(rowWithKeysOmitted: Omit<RowShape, OmitKeys>): RowShape;
	/**
	 * Updates a row in the table with the provided data.
	 *
	 * @param data - The data to update the row with, conforming to the RowShape interface.
	 * @returns The updated row data if the update is successful, or `null` if it fails.
	 * @throws Will throw an error if the primary column name is "ROWID".
	 */
	update(data: RowShape): RowShape | null;
	/**
	 * Updates rows in the table that match the specified conditions with the provided data.
	 *
	 * @param data - An object containing the data to update. The keys should correspond to the column names.
	 * @param conditions - An object containing the conditions to match the rows that need to be updated. The keys should correspond to the column names.
	 * @returns An array of updated rows.
	 */
	updateWhere(data: Partial<RowShape>, conditions: Partial<RowShape>): RowShape[];
	/**
	 * Updates all rows in the table with the provided data and returns the updated rows.
	 *
	 * @param data - An object containing the partial data to update the rows with.
	 *               The keys of this object should match the column names of the table.
	 * @returns An array of updated rows.
	 */
	updateAll(data: Partial<RowShape>): RowShape[];
	/**
	 * Checks if an entry with the specified ID exists in the table.
	 *
	 * @param id - The ID of the entry to check for existence.
	 * @returns `true` if an entry with the specified ID exists, otherwise `false`.
	 */
	exists(id: ColumnValue): boolean;
	/**
	 * Checks if a row exists in the table that matches the given conditions.
	 *
	 * @param conditions - An object representing the conditions to match against.
	 *                      Each key-value pair represents a column and the value to match.
	 * @returns `true` if a matching row is found, otherwise `false`.
	 */
	existsWhere(conditions: Partial<RowShape>): boolean;
	/**
	 * Counts the number of rows in the table that match the specified conditions.
	 * @param conditions The shape of data to count rows by
	 * @returns The number of rows that match the specified conditions.
	 */
	countWhere(conditions: Partial<RowShape>): number;
	/**
	 * Finds a single row by its unique identifier.
	 *
	 * @param id - The unique identifier of the row to find.
	 * @returns The row shape if found, otherwise `null`.
	 */
	findOneById(id: ColumnValue): RowShape | null;
	/**
	 * Finds a single row by its unique identifier. Throws an error if no row is found.
	 *
	 * @param id - The unique identifier of the row to find.
	 * @returns The row shape if found, otherwise `null`.
	 *
	 * @throws {Error} If no row is found with the specified ID.
	 */
	findOneByIdOrFail(id: ColumnValue): RowShape;
	/**
	 * Retrieves all rows from the table.
	 *
	 * @returns {RowShape[]} An array of all rows in the table.
	 */
	findAll(): RowShape[];
	/**
	 * Finds a single row in the table that matches the specified conditions.
	 *
	 * @param conditions - An object representing the conditions to match. The keys are column names and the values are the values to match.
	 * @returns The first row that matches the conditions, or `null` if no matching row is found.
	 */
	findOneWhere(conditions: Partial<RowShape>): RowShape | null;
	/**
	 * Finds a single row in the table that matches the specified conditions. Throws an error if no row is found.
	 *
	 * @param conditions - An object representing the conditions to match. The keys are column names and the values are the values to match.
	 * @returns The first row that matches the conditions, or `null` if no matching row is found.
	 *
	 * @throws {Error} If no row is found that matches the specified conditions.
	 */
	findOneWhereOrFail(conditions: Partial<RowShape>): RowShape;
	/**
	 * Finds all rows in the table that match the specified conditions.
	 *
	 * @param conditions - An object containing key-value pairs that represent the conditions for the query.
	 *                     The keys are column names and the values are the values to match.
	 * @returns An array of rows that match the specified conditions.
	 */
	findAllWhere(conditions: Partial<RowShape>): RowShape[];
	/**
	 * Deletes a row
	 *
	 * @param row - The row in the table to delete
	 * @returns The updated row data if the update is successful, or `null` if it fails.
	 * @throws Will throw an error if the primary column name is "ROWID".
	 */
	delete(data: RowShape): void;
	/**
	 * Deletes a record from the table by its ID.
	 *
	 * @param id - The unique identifier of the record to be deleted.
	 * @returns void
	 */
	deleteById: (id: ColumnValue) => void;
	/**
	 * Deletes all rows in the table that match the specified conditions.
	 *
	 * @param conditions - An object representing the conditions to match. The keys are column names and the values are the values to match.
	 */
	deleteWhere(conditions: Partial<RowShape>): void;
	/**
	 * Deletes all rows in the table.
	 */
	deleteAll(): void;
	/**
	 * Retrieves the size of the table.
	 *
	 * @deprecated Use `count()` instead.
	 * @returns {number} The number of rows in the table.
	 * @throws {Error} If the row count query fails.
	 */
	size(): number;
	/**
	 * Retrieves the size of the table.
	 *
	 * @returns {number} The number of rows in the table.
	 * @throws {Error} If the row count query fails.
	 */
	count(): number;
	save(wrapped: WrappedRow<RowShape>): void;
	toString(): string;
	get name(): string;
	get db(): Database;
	get columns(): string[];
	get schema(): string;
	get idColumnName(): string;
}
declare class PrequelKVStore {
	private table;
	/**
	 * Creates a new instance of the PrequelKVStore class.
	 * @param pq The Prequel instance.
	 * @param tableName The name of the table.
	 */
	constructor(pq: Prequel, tableName: string);
	/**
	 * Inserts or updates a key-value pair in the store.
	 * @param key The key.
	 * @param value The value.
	 */
	put<T>(key: string, value: T): void;
	/**
	 * Retrieves the value associated with the specified key from the store.
	 * @param key The key.
	 * @returns The value associated with the key, or null if the key is not found.
	 */
	get<T>(key: string): T | null;
	/**
	 * Checks if a key exists in the table.
	 * @param key - The key to check.
	 * @returns `true` if the key exists, `false` otherwise.
	 */
	has(key: string): boolean;
	/**
	 * Get an array of all of the keys in the KV store
	 * @returns The keys in the store.
	 */
	keys(): string[];
	/**
	 * Get an array of all of the values in the KV store
	 * @returns The values in the store.
	 */
	values(): unknown[];
}
/**
 * Represents a Prequel database instance.
 */
export declare class Prequel {
	db: Database;
	kv: PrequelKVStore;
	tables: Record<string, Table<unknown>>;
	query: typeof Database.prototype.query;
	uncachedQuery: typeof Database.prototype.prepare;
	/**
	 * Creates a new Prequel database instance.
	 * @param filename The filename of the database. Defaults to ":memory:" if not provided.
	 */
	constructor(filename?: string);
	/**
	 * Enables Write-Ahead Logging (WAL) mode for the database.
	 */
	enableWAL(): void;
	/**
	 * Closes the database connection.
	 */
	close(): void;
	/**
	 * Checks if a table with the given name exists in the database.
	 * @param name The name of the table.
	 * @returns `true` if the table exists, `false` otherwise.
	 */
	hasTable(name: string): boolean;
	/**
	 * Creates a new table in the database.
	 * @param name The name of the table.
	 * @param cols The column definitions of the table.
	 * @returns The created table.
	 * @throws {SchemaError} If a table with the same name already exists.
	 */
	table<RowShape>(name: string, cols: ColumnShorthandMap<RowShape>): Table<RowShape>;
	/**
	 * Attaches an existing table to the database.
	 * @param table The table to attach.
	 * @throws {SchemaError} If a table with the same name already exists.
	 */
	attachTable(table: Table<unknown>): void;
	/**
	 * Executes a function within a database transaction.
	 * If the function throws an error, the transaction is rolled back.
	 * Otherwise, the transaction is committed.
	 * @param transactionFn The function to execute within the transaction.
	 * @returns `true` if the transaction was committed successfully, otherwise `false`.
	 * @throws The error thrown by the transaction function if the transaction fails.
	 */
	transaction<T>(transactionFn: () => T): T;
}

export {};
